# Neil Shephard 8th July 2020 empirical code for "An estimator for predictive regression:
# reliable inference for financial economics"

install.packages("quantmod")
library(quantmod)
library(qrmtools)
library(MASS)

   # tickers for major companies used in this analysis

symbols = c("ABT", "ARG","ABBV", "ACN", "ACE", "ADBE", "ADT", "AAP", "AES", "AET", "AFL", "AMG", "A", "GAS", "APD", "ARG", 
            "AKAM", "AA", "AGN", "ALXN", "ALLE", "ADS", "ALL", "ALTR", "MO", "AMZN", "AEE", "AAL", "AEP", "AXP", "AIG", 
            "AMT", "AMP", "ABC", "AME", "AMGN", "APH", "APC", "ADI", "AON", "APA", "AIV", "AMAT", "ADM", "AIZ", "T", 
            "ADSK", "ADP", "AN", "AZO", "AVGO", "AVB", "AVY", "BHI", "BLL", "BAC", "BK", "BCR", "BXLT", "BAX", "BBT", 
            "BDX", "BBBY", "BRK-B", "BBY", "BLX", "HRB", "BA", "BWA", "BXP", "BSK", "BMY", "BRCM", "BF-B", "CHRW", "CA", 
            "CVC", "COG", "CAM", "CPB", "COF", "CAH", "HSIC", "KMX", "CCL", "CAT", "CBG", "CBS", "CELG", "CNP", "CTL", 
            "CERN", "CF", "SCHW", "CVX", "CMG", "CB", "CI", "XEC", "CINF", "CTAS", "CSCO", "C", "CTXS", "CLX", 
            "CME", "CMS", "COH", "KO", "CCE", "CTSH", "CL", "CMCSA", "CMA", "CAG", "COP", "CNX", "ED", "STZ", 
            "GLW", "COST", "CCI", "CSX", "CMI", "CVS", "DHI", "DHR", "DRI", "DVA", "DE", "DLPH", "DAL", "XRAY", "DVN", 
            "DO", "DTV", "DFS", "DISCA", "DISCK", "DG", "DLTR", "D", "DOV", "DOW", "DPS", "DTE", "DD", "DUK", "DNB", 
            "ETFC", "EMN", "ETN", "EBAY", "ECL", "EIX", "EA", "EMC", "EMR", "ENDP", "ESV", "ETR", "EOG", "EQT", 
            "EFX", "EQIX", "EQR", "ESS", "EL", "ES", "EXC", "EXPE", "EXPD", "ESRX", "XOM", "FFIV", "FB", "FAST", "FDX", 
            "FIS", "FITB", "FSLR", "FE", "FSIV", "FLIR", "FLS", "FLR", "FMC", "FTI", "F", "FOSL", "BEN", "FCX", "FTR", 
            "GME", "GPS", "GRMN", "GD", "GE", "GGP", "GIS", "GM", "GPC", "GNW", "GILD", "GS", "GT", "GOOGL", "GOOG", 
            "GWW", "HAL", "HBI", "HOG", "HAR", "HRS", "HIG", "HAS", "HCA", "HCP", "HCN", "HP", "HES", "HPQ", 
            "HD", "HON", "HRL", "HSP", "HST", "HCBK", "HUM", "HBAN", "ITW", "IR", "INTC", "ICE", "IBM", "IP", "IPG", 
            "IFF", "INTU", "ISRG", "IVZ", "IRM", "JEC", "JBHT", "JNJ", "JCI", "JOY", "JPM", "JNPR", "KSU", "K", "KEY", 
            "GMCR", "KMB", "KIM", "KMI", "KLAC", "KSS", "KRFT", "KR", "LB", "LLL", "LH", "LRCX", "LM", "LEG", "LEN", 
            "LVLT", "LUK", "LLY", "LNC", "LLTC", "LMT", "L", "LOW", "LYB", "MTB", "MAC", "M", "MNK", "MRO", "MPC", 
            "MAR", "MMC", "MLM", "MAS", "MA", "MAT", "MKC", "MCD", "MHFI", "MCK", "MJN", "MMV", "MDT", "MRK", "MET", 
            "KORS", "MCHP", "MU", "MSFT", "MHK", "TAP", "MDLZ", "MON", "MNST", "MCO", "MS", "MOS", "MSI", "MUR", 
            "MYL", "NDAQ", "NOV", "NAVI", "NTAP", "NFLX", "NWL", "NFX", "NEM", "NWSA", "NEE", "NLSN", 
            "NKE", "NI", "NE", "NBL", "JWN", "NSC", "NTRS", "NOC", "NRG", "NUE", "NVDA", "ORLY", "OXY", 
            "OMC", "OKE", "ORCL", "OI", "PCAR", "PLL", "PH", "PDCO", "PAYX", "PNR", "PBCT", "POM", "PEP", 
            "PKI", "PRGO", "PFE", "PCG", "PM", "PSX", "PNW", "PXD", "PBI", "PCL", "PNC", "RL", "PPG", "PPL", 
            "PX", "PCP", "PCLN", "PFG", "PG", "PGR", "PLD", "PRU", "PEG", "PSA", "PHM", "PVH", "QRVO", "PWR", 
            "QCOM", "DGX", "RRC", "RTN", "O", "RHT", "REGN", "RF", "RSG", "RAI", "RHI", "ROK", "COL", "ROP", 
            "ROST", "RLC", "R", "CRM", "SNDK", "SCG", "SLB", "SNI", "STX", "SEE", "SRE", "SHW", "SIAL", "SPG", 
            "SWKS", "SLG", "SJM", "SNA", "SO", "LUV", "SWN", "SE", "STJ", "SWK", "SPLS", "SBUX", "HOT", "STT", 
            "SRCL", "SYK", "STI", "SYMC", "SYY", "TROW", "TGT", "TEL", "TE", "TGNA", "THC", "TDC", "TSO", "TXN", 
            "TXT", "HSY", "TRV", "TMO", "TIF", "TWX", "TWC", "TJK", "TMK", "TSS", "TSCO", "RIG", "TRIP", "FOXA", 
            "TSN", "TYC", "UA", "UNP", "UNH", "UPS", "URI", "UTX", "UHS", "UNM", "URBN", "VFC", "VLO", "VAR", "VTR", 
            "VRSN", "VZ", "VRTX", "VIAB", "VNO", "VMC", "WMT", "WBA", "DIS", "WM", "WAT", "ANTM", "WFC", "WDC", 
            "WU", "WY", "WHR", "WFM", "WMB", "WEC", "WYN", "WYNN", "XEL", "XRX", "XLNX", "XL", "XYL", "YHOO", "YUM", 
            "ZBH", "ZION", "ZTS"); 

getSymbols("SPY",from='2018-08-01',to='2020-08-04',verbose = TRUE); head(SPY);  # download S&P500 ETF data
XdataM = (to.weekly(SPY))$SPY.Adjusted  # move to weekly returns, included dividends
x = data.matrix(100*diff(XdataM)/lag(XdataM))[-1]; # compute arithmetic returns

modelT = fitdistr(x, "t", start = list(m=mean(X),s=sd(X), df=3), lower=c(-1, 0.001,1));  # ML of student t

w = 1.0/sqrt(1.0+((x-mean(x))*(x-mean(x))));
Xhat = sum(w*x)/sum(w); Xbar = mean(x);
absX = mean(abs(x-Xbar))*10*(n^0.2);

#aX = mean(X);
#X = X - mean(X);
#S = sign(X) - mean(sign(X));
#absX = mean(abs(X))*10*(n^0.2);  #10

n =length(x);

aOut = getSymbols(symbols,from='2018-08-01',to='2020-08-04',verbose = TRUE); # download individual companies
b = length(aOut); # number of companies

mAns = data.frame(matrix(0,b,9));  # define space to store results

for (i in 1:b){ # loop over individual companies
#for (i in 1:30){ # loop over 6 companies
  print(c(i,b,n));
  print(aOut[i]);
  YdataM = to.weekly(eval(as.symbol(aOut[i])))[,6]; # move to weekly returns
  print(c(i,b,n));

  y = data.matrix(100*diff(YdataM)/lag(YdataM))[-1]; # compute arithmetic returns
  aY = mean(y);
#  y = y-mean(y); # center
  
  print(aOut[i]);
  print(c(length(x),length(y)));
  
  if (length(x) == length(y)){ # only look at companies where data download has nothing missing
    
    Yhat = sum(w*y)/sum(w); Ybar = mean(y);
    betahat   = sum(w*(x-Xhat)*(y-Yhat))/sum(w*(x-Xhat)*(x-Xhat));
    uhat = (y - Yhat) - (x-Xhat)*betahat;
    uhat = uhat*(abs(x-Xbar) < absX);
    wX = (w/mean(w)) - ((Xhat-Xbar)*(x-Xhat)*w/mean(w*(x-Xhat)*(x-Xhat)));
    betahat.se = sqrt((1.0/(n-2))*mean(w*w*(x-Xhat)*(x-Xhat)*uhat*uhat)/(mean(w*(x-Xhat)*(x-Xhat))^2));
    alphahat = Yhat - (Xhat-Xbar)*(betahat);
    alphahat.se = sqrt((1.0/(n-2))*mean(wX*wX*uhat*uhat));
    
    betaOLS = sum((x-Xbar)*(y-Ybar))/sum((x-Xbar)*(x-Xbar));
    utilda = (y - Ybar) - (x-Xbar)*betaOLS;
    betaOLS.se = sqrt((1.0/(n-2))*mean((x-Xbar)*(x-Xbar)*utilda*utilda)/(mean((x-Xbar)*(x-Xbar))^2));
    alphaOLS = Ybar;
    alphaOLS.se = sqrt((1.0/(n-2))*mean(utilda*utilda));
    
    
#     betaOLS = mean(X*Y)/mean(X*X);          alphaOLS = aY - (betaOLS*aX);
#     betahat = mean(S*Y)/mean(S*X);          alphahat = aY - (betahat*aX);
#     WX= 1.0 - aX*X/mean(X*X);
#     W = 1.0 - aX*S/mean(S*X);
  
#     U = Y - betaOLS*X;                         betaOLS.se = sqrt(1.0/(n-2))*sqrt(mean(X*X*U*U)/(mean(X*X)^2));
#                                                alphaOLS.se= sqrt(1.0/(n-2))*sqrt(mean(WX*WX*U*U));
#     V = Y - betahat*X;  V = V*(abs(X) < absX); betahat.se = sqrt(1.0/(n-2))*sqrt(mean(V*V*S*S)/(mean(S*X)^2));
#                                                alphahat.se= sqrt(1.0/(n-2))*sqrt(mean(W*W*V*V));

     plot(x,y,pch=16,main="Stock versus S&P500 daily returns",xlab="S&P500",ylab=aOut[i])
     abline(a=0,b=betaOLS,col="red")
     abline(a=0,b=betahat,col="blue")
     
#     mAns[i,] = c(i,betaOLS,betaOLS.se,betahat,betahat.se,alphaOLS,alphaOLS.se,alphahat,alphahat.se);
     mAns[i,] = c(i,betahat,betaOLS,betahat.se,betaOLS.se,alphahat,alphaOLS,alphahat.se,alphaOLS.se);
  }
  else{mAns[i,] = NA;
  }
  row.names(mAns)[i] = aOut[i];
}

mAns1 = mAns[!is.na(mAns[,1]),];
aOut1 = aOut[!is.na(mAns[,1])];

pdf("spthetahat.pdf");
 plot(mAns1[,3],mAns1[,2],main="beta estimates",xlab="OLS",ylab="beta hat",pch=16,cex=0.4,xlim=c(-0.5,3.0),ylim=c(-0.5,3.0));
 abline(0,1,col="blue");
 abline(lm(mAns1[,2] ~ mAns1[,3]),col="red")
 summary(lm(mAns1[,2] ~ mAns1[,3]));
dev.off();

pdf("spSE.pdf");
 plot(mAns1[,5],mAns1[,4],main="SE estimates",xlab="OLS",ylab="beta hat",pch=16,cex=0.4,xlim=c(0,1.2),ylim=c(0,1.2));
 abline(0,1,col="blue");
 abline(lm(mAns1[,4] ~ mAns1[,5]),col="red")
 summary(lm(mAns1[,4] ~ mAns1[,5]))
dev.off();

colMeans(mAns1);
for (i in (2:9)){
 print(quantile(mAns1[,i],probs=c(0.1,0.5,0.9)));
}

tBhat= (mAns1[,2]-1.4)/mAns1[,4]; id  = (tBhat>1.64);
tOLS = (mAns1[,3]-1.4)/mAns1[,5]; id1 = (tOLS>1.64);

sum(id)
sum(id1)
sum(id*id1)

#pdf("TtestHigh.pdf");
#  plot(tOLS,tBhat,main="T plots",xlab="hat T_LS",ylab="hat T",pch=16,cex=0.4,xlim=c(-16,5),ylim=c(-16,5));
#  abline(0,1,col="blue");
#  abline(lm(tBhat ~ tOLS),col="red")
#  abline(v=1.64,col="green",lwd=3)
#  abline(h=1.64,col="green",lwd=3,lty="dotted")
#  summary(lm(tBhat ~ tOLS))
#dev.off();

pdf("spbigBeta.pdf");
id = (((mAns1[,2]-1.4)/mAns1[,4])> 1.96); 
plot(mAns1[id,3],mAns1[id,2],main="High beta stocks: selected using beta hat",
     xlab="OLS",ylab="beta hat",pch=16,cex=0.4,ylim=c(1.0,3.0),xlim=c(1.0,3.0));
text(mAns1[id,3],mAns1[id,2],labels=aOut1[id],cex=1.2, font=2)
abline(0,1,col="blue");
dev.off();

pdf("spbigBeta2.pdf");
id1 = ((mAns1[,3]-1.4)/mAns1[,5]> 1.96); 
plot(mAns1[id1,3],mAns1[id1,2],main="High betas stocks: selected using OLS",
     xlab="OLS",ylab="beta hat",pch=16,cex=0.4,ylim=c(1.0,3.0),xlim=c(1.0,3.0))
text(mAns1[id1,3],mAns1[id1,2],labels=aOut1[id1],cex=1.2, font=2)
abline(0,1,col="blue");
dev.off();


tBhat= (mAns1[,2]-0.8)/mAns1[,4]; id  = (tBhat< -1.64);
tOLS = (mAns1[,3]-0.8)/mAns1[,5]; id1 = (tOLS < -1.64);

sum(id)
sum(id1)
sum(id*id1)

#pdf("TtestLow.pdf");
#  plot(tOLS,tBhat,main="T plots",xlab="hat T_LS",ylab="hat T",pch=16,cex=0.4,xlim=c(-6,10),ylim=c(-6,10));
#  abline(0,1,col="blue");
#  abline(lm(tBhat ~ tOLS),col="red");
#  abline(v=-1.64,col="green",lwd=3);
#  abline(h=-1.64,col="green",lwd=3,lty="dotted");
#  summary(lm(tBhat ~ tOLS));
#dev.off();

pdf("spsmallBeta.pdf");
plot(mAns1[id,3],mAns1[id,2],main="Low beta stocks: selected by beta hat",
     xlab="OLS",ylab="beta hat",pch=16,cex=0.4,ylim=c(-0.2,1.0),xlim=c(-0.2,1.0));
text(mAns1[id,3],mAns1[id,2],labels=aOut1[id],cex=1.2, font=2)
abline(0,1,col="blue");
#dev.off();

#pdf("spsmallBeta2.pdf");
plot(mAns1[id1,3],mAns1[id1,2],main="Low betas stocks: selected by OLS",
     xlab="OLS",ylab="beta hat",pch=16,cex=0.4,ylim=c(-0.2,1.0),xlim=c(-0.2,1.0))
text(mAns1[id1,3],mAns1[id1,2],labels=aOut1[id1],cex=1.2, font=2)
abline(0,1,col="blue");
dev.off();

pdf("summaryOLS.pdf");
  plot(mAns1[,5],mAns1[,3],main="beta OLS against SE",xlab="SE",ylab="OLS",pch=16,cex=0.4,xlim=c(0.0,1.0),ylim=c(0.0,3.0));
  abline(a=1.4,b=qnorm(0.95),col="red");
  abline(a=0.8,b=-qnorm(0.95),col="green");
dev.off();

pdf("summaryBetahat.pdf");
  plot(mAns1[,4],mAns1[,2],main="beta hat against SE",xlab="SE",ylab="beta hat",pch=16,cex=0.4,xlim=c(0.0,1.0),ylim=c(0.0,3.0));
  abline(a=1.4,b=qnorm(0.95),col="red");
  abline(a=0.8,b=-qnorm(0.95),col="green");
dev.off();

pdf("summaryAlphaOLS.pdf");
  plot(mAns1[,9],mAns1[,7],main="alpha OLS against SE",xlab="SE",ylab="alpha OLS",xlim=c(0,2),ylim=c(-3,3),pch=16,cex=0.4);
  abline(a=0.0,b=qnorm(0.975),col="red");
  abline(a=0.0,b=-qnorm(0.975),col="red");
dev.off();

pdf("summaryAlphahat.pdf");
  plot(mAns1[,8],mAns1[,6],main="alpha hat against SE",xlab="SE",ylab="alpha hat",xlim=c(0,2),ylim=c(-3,3),pch=16,cex=0.4);
  abline(a=0.0,b=qnorm(0.975),col="red");
  abline(a=0.0,b=-qnorm(0.975),col="red");
dev.off();

plot(mAns1[,6],mAns1[,7])
plot(mAns1[,8],mAns1[,9])

