# Neil Shephard 7th July 2020 simulation code for "An estimator for predictive regression:
# reliable inference for financial economics"

iRep=3000; n=100; theta = 0.5; nu=4.4; s2 = 2.0;
library(goftest);

set.seed(111); 

mRes = matrix(0,28,3);

for (mm in (1:4)){
  if (mm==1){ n=100; s2 = 2.0;} # plots against nu
  if (mm==2){ n=250; s2 = 2.0;}
  if (mm==3){ n=100; s2 = 4.0;}
  if (mm==4){ n=250; s2 = 4.0;}


for (kk in (1:(2+28))){
#for (kk in (1:1)){
  
  if (kk==1) nu=2.4;  # QQ plots
  if (kk==2) nu=4.4;  # QQ plots
  
  iRep=3000;
  
  if (kk>2){          # plots against nu
     nu= 2.4 + ((kk-3)/4)^2;
     iRep=500000;
  }
  
  xRes = matrix(0,iRep,2);
  
  for (i in (1:iRep)){
    x = 0.21+3.24*sqrt(((nu-2)/nu))*rt(n,nu);
    y = rnorm(n,theta*x,s2);

    w = 1.0/sqrt(1.0+((x-mean(x))*(x-mean(x))));
    Yhat = sum(w*y)/sum(w); Ybar = mean(y);
    Xhat = sum(w*x)/sum(w); Xbar = mean(x);
    absX = mean(abs(x-Xbar))*10*(n^0.2);
    
    thetahat   = sum(w*(x-Xhat)*(y-Yhat))/sum(w*(x-Xhat)*(x-Xhat));
    uhat = (y - Yhat) - thetahat*(x-Xhat);
    uhat = uhat*(abs(x-Xbar) < absX);
    varhat = (1.0/(n-2))*mean(w*w*(x-Xhat)*(x-Xhat)*uhat*uhat)/(mean(w*(x-Xhat)*(x-Xhat))^2);
    
    thetaLS = sum((x-Xbar)*(y-Ybar))/sum((x-Xbar)*(x-Xbar));
    utilda = (y - Ybar) - thetaLS*(x-Xbar);
    varLS = (1.0/(n-2))*mean((x-Xbar)*(x-Xbar)*utilda*utilda)/(mean((x-Xbar)*(x-Xbar))^2);
    
    xRes[i,c(1,2)] = c((thetahat-theta)/sqrt(varhat),(thetaLS-theta)/sqrt(varLS));
  }

  if (kk>2){  # Cramer-von-Mises stats for plots against nu
    cvm1 = cvm.test(xRes[,1], "pnorm",mean=0,sd=1);
    cvm2 = cvm.test(xRes[,2], "pnorm",mean=0,sd=1);

    mRes[(kk-2),] = c(nu,cvm1$statistic,cvm2$statistic);
  }

  if (kk==1 && mm==1){  # QQ plots
   pdf("QQhat.pdf"); 
     qqnorm(xRes[,1],pch=16,xlim=range(xRes[,1]),main="QQ plot for hat{T}"); 
     abline(0,1,col="red");
   dev.off();

   pdf("QQOLS.pdf"); 
     qqnorm(xRes[,2],pch=16,xlim=range(xRes[,2]),main="QQ plot for hat{T}_{LS}"); 
     abline(0,1,col="red");
   dev.off();
  }
  
  if (kk==2 && mm==1){
    pdf("QQhat7.pdf"); 
      qqnorm(xRes[,1],pch=16,xlim=range(xRes[,1]),main="QQ plot for hat{T}"); 
      abline(0,1,col="red");
    dev.off();
    
    pdf("QQOLS7.pdf"); 
      qqnorm(xRes[,2],pch=16,xlim=range(xRes[,2]),main="QQ plot for hat{T}_{LS}"); 
      abline(0,1,col="red");
    dev.off();
  }
}  
  
mRep = matrix(0,5000,1); # quantiles for plots against nu
for (i in (1:5000)){
      mRep[i] = cvm.test(rnorm(iRep,0,1), "pnorm",mean=0,sd=1)$statistic
}

if (mm==1){pdf("CVM100s2.pdf");  # plots against nu
  }

if (mm==2){pdf("CVM250s2.pdf");
}

if (mm==3){pdf("CVM100s4.pdf");
}

if (mm==4){pdf("CVM250s4.pdf");
}

  matplot(mRes[,1],mRes[,2:3],type="l",xlab="nu",ylab="Test Statistic",xlim=c(2,50),log="xy",lwd=2)
  abline(h=quantile(mRep,probs=c(0.5)),col="green",lwd=2); #ylim=c(0.04,100),
  abline(h=quantile(mRep,probs=c(0.95)),col="green",lwd=2); #ylim=c(0.04,100),
  abline(h=quantile(mRep,probs=c(0.99)),col="green",lwd=2); #ylim=c(0.04,100),

dev.off();

}


  



# case where Var(X_1) = infty

mRes = matrix(0,8,5);

for (p in (1:4))
{

iRep=500000; theta = 1.0; 
xRes = matrix(0,iRep,5);

if (p==1) n=100
if (p==2) n=250;
if (p==3) n=1000;
if (p==4) n=10000;


for (k in (1:8)){
  nu=1.1 + ((k-1)*1.3)/8;
  scale = mean(abs(rt(n,nu)));

  for (i in (1:iRep)){
    x = 0.21+3.24*rt(n,nu)/scale;
    y = rnorm(n,theta*x,2.0);
    
    w = 1.0/sqrt(1.0+((x-mean(x))*(x-mean(x))));
    Yhat = sum(w*y)/sum(w); Ybar = mean(y);
    Xhat = sum(w*x)/sum(w); Xbar = mean(x);
    absX = mean(abs(x-Xbar))*10*(n^0.2);
    
    thetahat   = sum(w*(x-Xhat)*(y-Yhat))/sum(w*(x-Xhat)*(x-Xhat));
    uhat = (y - Yhat) - thetahat*(x-Xhat);
    uhat = uhat*(abs(x-Xbar) < absX);
    varhat = (1.0/(n-2))*mean(w*w*(x-Xhat)*(x-Xhat)*uhat*uhat)/(mean(w*(x-Xhat)*(x-Xhat))^2);
    
    thetaLS = sum((x-Xbar)*(y-Ybar))/sum((x-Xbar)*(x-Xbar));
    utilda = (y - Ybar) - thetaLS*(x-Xbar);
    varLS = (1.0/(n-2))*mean((x-Xbar)*(x-Xbar)*utilda*utilda)/(mean((x-Xbar)*(x-Xbar))^2);
    
    xRes[i,1] = c((thetahat-theta)/sqrt(varhat));
    
  }
  cvm1 = cvm.test(xRes[,1], "pnorm",mean=0,sd=1)
  mRes[k,c(1,(p+1))] = c(nu,cvm1$statistic)
  
  print(mRes[k,c(1,(p+1))])
}
}

mRep = matrix(0,5000,1);
for (i in (1:5000)){
  mRep[i] = cvm.test(rnorm(iRep,0,1), "pnorm",mean=0,sd=1)$statistic
}

pdf("CVMtails.pdf");
  matplot(mRes[,1],mRes[,-1],type="l",xlab="nu",ylab="Test Statistic",xlim=c(1.1,2.4),log="xy",lwd=4);
  abline(h=quantile(mRep,probs=c(0.5)),col="green",lwd=2); 
  abline(h=quantile(mRep,probs=c(0.95)),col="green",lwd=2); 
  abline(h=quantile(mRep,probs=c(0.99)),col="green",lwd=2); 
dev.off();

